# Copyright 2017-2018 Aaron C. Prunty
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#        
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==============================================================================

import sys
import argparse
import vezda.LSM
import vezda.TELSM

def cli():
    parser = argparse.ArgumentParser()
    parser.add_argument('--method', type=str, choices=['lsm', 'telsm'], required=True,
                        help='''Specify whether to use the classical linear sampling method 'lsm'
                        or total-energy linear sampling method 'telsm'.''')
    parser.add_argument('--regPar', '--alpha', type=float,
                        help='Specify the value of the regularization parameter')
    parser.add_argument('--timeShift', '--tau', type=float,
                        help='specify the value of the time-shift parameter')
    parser.add_argument('--medium', type=str, default='constant', choices=['constant', 'variable'],
                        help='''Specify whether the background medium is constant or variable
                        (inhomogeneous). If argument is set to 'constant', the velocity defined in
                        the required 'pulseFun.py' file is used. Default is set to 'constant'.''')
    args = parser.parse_args()
    
    if args.regPar is not None and args.regPar >= 0:
        # alpha is the value of the regularization parameter
        alpha = args.regPar
    elif args.regPar is not None and args.regPar < 0:
        sys.exit('''
              Error: Optional argument '--regPar/--alpha' cannot be negative. 
              The regularization parameter must be greater than or equal to zero.
              ''')
    elif args.regPar is None:
        alpha = 0
        
    if args.method == 'lsm':
        if args.timeShift is None:
            userResponded = False
            print('''
                  Warning: Required argument '--timeShift/--tau' for classical LSM was 
                  not specified. The parameter will be set to the default value of 0.
                  What would you like to do?
                  Enter '1' to specify a value for the parameter. (Default)
                  Enter '2' to proceed with the default value.
                  Enter 'q/quit' to abort the calculation.
                  ''')
            while userResponded == False:
                answer = input('Action: ')
                if answer == '' or answer == '1':
                    tau = float(input('Please specify the value of the time-shift parameter: timeShift = '))
                    print('Proceeding with classical LSM using regPar = %s and timeShift = %s' %(alpha, tau))
                    userResponded = True
                    break
                elif answer == '2':
                    tau = 0
                    print('Proceeding with classical LSM using regPar = %s and timeShift = %s' %(alpha, tau))
                    userResponded = True
                    break
                elif answer == 'q' or answer == 'quit':
                    sys.exit('Exiting program.')
                else:
                    print('Invalid response. Please enter \'1\', \'2\', or \'q/quit\'.')
                   
        elif args.timeShift is not None:
            tau = args.timeShift
        
        vezda.LSM.solver(args.medium, alpha, tau)
    
    elif args.method == 'telsm':
        vezda.TELSM.solver(args.medium, alpha)