# Copyright 2017-2018 Aaron C. Prunty
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#        
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#==============================================================================

import sys
import argparse
import numpy as np

def cli():
    parser = argparse.ArgumentParser()
    parser.add_argument('--xaxis', type=str, required=True,
                        help='''specify the endpoint values of the sampling
                        grid along the x-axis and the number of sampling points
                        in between. Syntax: --xaxis=start,stop,num.''')
    parser.add_argument('--yaxis', type=str, required=True,
                        help='''specify the endpoint values of the sampling
                        grid along the y-axis and the number of sampling points
                        in between. Syntax: --yaxis=start,stop,num.''')
    parser.add_argument('--zaxis', type=str, required=False, default=None,
                        help='''specify the endpoint values of the sampling
                        grid along the z-axis and the number of sampling points
                        in between. Syntax: --zaxis=start,stop,num.''')
    args = parser.parse_args()
    
    if args.zaxis == None:
        # set grid along x-axis
        xaxis = args.xaxis.split(',')
        if len(xaxis) != 3:
            sys.exit('''
                     Error: Must specify three values when using --xaxis parameter.
                     Syntax: --xaxis=start,stop,num
                     ''')
        if float(xaxis[0]) < float(xaxis[1]) and int(xaxis[2]) > 0:
            x = np.linspace(float(xaxis[0]), float(xaxis[1]), int(xaxis[2]))
        elif float(xaxis[0]) >= float(xaxis[1]):
            sys.exit('''
                     Error: Starting x-value of the sampling grid must be less
                     than the ending x-value of the sampling grid.
                     ''')
        elif int(xaxis[2]) <= 0:
            sys.exit('''
                     Error: Number of sampling points along the x-axis
                     must be greater than zero.
                     ''')
        
        # set grid along y-axis
        yaxis = args.yaxis.split(',')
        if len(yaxis) != 3:
            sys.exit('''
                     Error: Must specify three values when using --yaxis parameter.
                     Syntax: --yaxis=start,stop,num
                     ''')
        if float(yaxis[0]) < float(yaxis[1]) and int(yaxis[2]) > 0:
            y = np.linspace(float(yaxis[0]), float(yaxis[1]), int(yaxis[2]))
        elif float(yaxis[0]) >= float(yaxis[1]):
            sys.exit('''
                     Error: Starting y-value of the sampling grid must be less
                     than the ending y-value of the sampling grid.
                     ''')
        elif int(yaxis[2]) <= 0:
            sys.exit('''
                     Error: Number of sampling points along the y-axis
                     must be greater than zero.
                     ''')
        
        print('Setting up two-dimensional sampling grid...')
        print('grid @ x-axis : start = ', float(xaxis[0]))
        print('grid @ x-axis : stop = ', float(xaxis[1]))
        print('grid @ x-axis : num = ', int(xaxis[2]))
        
        print('grid @ y-axis : start = ', float(yaxis[0]))
        print('grid @ y-axis : stop = ', float(yaxis[1]))
        print('grid @ y-axis : num = ', int(yaxis[2]))
        np.savez('grid.npz', ndim=2, x=x, y=y)
    
    elif args.zaxis != None:
        # set grid along x-axis
        xaxis = args.xaxis.split(',')
        if len(xaxis) != 3:
            sys.exit('''
                     Error: Must specify three values when using --xaxis parameter.
                     Syntax: --xaxis=start,stop,num
                     ''')
        if float(xaxis[0]) < float(xaxis[1]) and int(xaxis[2]) > 0:
            x = np.linspace(float(xaxis[0]), float(xaxis[1]), int(xaxis[2]))
        elif float(xaxis[0]) >= float(xaxis[1]):
            sys.exit('''
                     Error: Starting x-value of the sampling grid must be less
                     than the ending x-value of the sampling grid.
                     ''')
        elif int(xaxis[2]) <= 0:
            sys.exit('''
                     Error: Number of sampling points along the x-axis
                     must be greater than zero.
                     ''')
        
        # set grid along y-axis
        yaxis = args.yaxis.split(',')
        if len(yaxis) != 3:
            sys.exit('''
                     Error: Must specify three values when using --yaxis parameter.
                     Syntax: --yaxis=start,stop,num
                     ''')
        if float(yaxis[0]) < float(yaxis[1]) and int(yaxis[2]) > 0:
            y = np.linspace(float(yaxis[0]), float(yaxis[1]), int(yaxis[2]))
        elif float(yaxis[0]) >= float(yaxis[1]):
            sys.exit('''
                     Error: Starting y-value of the sampling grid must be less
                     than the ending y-value of the sampling grid.
                     ''')
        elif int(yaxis[2]) <= 0:
            sys.exit('''
                     Error: Number of sampling points along the y-axis
                     must be greater than zero.
                     ''')
            
        # set grid along z-axis
        zaxis = args.zaxis.split(',')
        if len(zaxis) != 3:
            sys.exit('''
                     Error: Must specify three values when using --zaxis parameter.
                     Syntax: --zaxis=start,stop,num
                     ''')
        if float(zaxis[0]) < float(zaxis[1]) and int(zaxis[2]) > 0:
            z = np.linspace(float(zaxis[0]), float(zaxis[1]), int(zaxis[2]))
        elif float(zaxis[0]) >= float(zaxis[1]):
            sys.exit('''
                     Error: Starting z-value of the sampling grid must be less
                     than the ending z-value of the sampling grid.
                     ''')
        elif int(zaxis[2]) <= 0:
            sys.exit('''
                     Error: Number of sampling points along the z-axis
                     must be greater than zero.
                     ''')
        
        print('Setting up three-dimensional sampling grid...')
        print('grid @ x-axis : start = ', float(xaxis[0]))
        print('grid @ x-axis : stop = ', float(xaxis[1]))
        print('grid @ x-axis : num = ', int(xaxis[2]))
        
        print('grid @ y-axis : start = ', float(yaxis[0]))
        print('grid @ y-axis : stop = ', float(yaxis[1]))
        print('grid @ y-axis : num = ', int(yaxis[2]))
        
        print('grid @ z-axis : start = ', float(zaxis[0]))
        print('grid @ z-axis : stop = ', float(zaxis[1]))
        print('grid @ z-axis : num = ', int(zaxis[2]))
        np.savez('grid.npz', ndim=3, x=x, y=y, z=z)
